package sun.text.normalizer;

import java.text.Normalizer;
import java.text.CharacterIterator;

public final class NormalizerBase implements Cloneable
{
    private char[] buffer;
    private int bufferStart;
    private int bufferPos;
    private int bufferLimit;
    private UCharacterIterator text;
    private NormalizerBase.Mode mode;
    private int options;
    private int currentIndex;
    private int nextIndex;
    public static final int UNICODE_3_2 = 32;
    public static final int DONE = -1;
    public static final NormalizerBase.Mode NONE;
    public static final NormalizerBase.Mode NFD;
    public static final NormalizerBase.Mode NFKD;
    public static final NormalizerBase.Mode NFC;
    public static final NormalizerBase.Mode NFKC;
    public static final NormalizerBase.QuickCheckResult NO;
    public static final NormalizerBase.QuickCheckResult YES;
    public static final NormalizerBase.QuickCheckResult MAYBE;
    private static final int MAX_BUF_SIZE_COMPOSE = 2;
    private static final int MAX_BUF_SIZE_DECOMPOSE = 3;
    public static final int UNICODE_3_2_0_ORIGINAL = 262432;
    public static final int UNICODE_LATEST = 0;
    
    public NormalizerBase(String s, NormalizerBase.Mode mode, int options) {
        this.buffer = new char[100];
        this.bufferStart = 0;
        this.bufferPos = 0;
        this.bufferLimit = 0;
        this.mode = NormalizerBase.NFC;
        this.options = 0;
        this.text = UCharacterIterator.getInstance(s);
        this.mode = mode;
        this.options = options;
    }
    
    public NormalizerBase(CharacterIterator characterIterator, NormalizerBase.Mode mode) {
        this(characterIterator, mode, 0);
    }
    
    public NormalizerBase(CharacterIterator characterIterator, NormalizerBase.Mode mode, int options) {
        this.buffer = new char[100];
        this.bufferStart = 0;
        this.bufferPos = 0;
        this.bufferLimit = 0;
        this.mode = NormalizerBase.NFC;
        this.options = 0;
        this.text = UCharacterIterator.getInstance((CharacterIterator)characterIterator.clone());
        this.mode = mode;
        this.options = options;
    }
    
    public Object clone() {
        try {
            NormalizerBase normalizerBase = (NormalizerBase)super.clone();
            normalizerBase.text = (UCharacterIterator)this.text.clone();
            if (this.buffer != null) {
                normalizerBase.buffer = new char[this.buffer.length];
                System.arraycopy(this.buffer, 0, normalizerBase.buffer, 0, this.buffer.length);
                return normalizerBase;
            }
            return normalizerBase;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    public static String compose(String s, boolean b, int n) {
        char[] value;
        char[] array;
        if (n == 262432) {
            String convert = NormalizerImpl.convert(s);
            value = new char[convert.length() * 2];
            array = convert.toCharArray();
        }
        else {
            value = new char[s.length() * 2];
            array = s.toCharArray();
        }
        UnicodeSet nx = NormalizerImpl.getNX(n);
        n &= 0xFFFFCF00;
        if (b) {
            n |= 0x1000;
        }
        int compose;
        while (true) {
            compose = NormalizerImpl.compose(array, 0, array.length, value, 0, value.length, n, nx);
            if (compose <= value.length) {
                break;
            }
            value = new char[compose];
        }
        return new String(value, 0, compose);
    }
    
    public static String decompose(String s, boolean b) {
        return decompose(s, b, 0);
    }
    
    public static String decompose(String s, boolean b, int n) {
        int[] array = { 0 };
        UnicodeSet nx = NormalizerImpl.getNX(n);
        if (n == 262432) {
            String convert = NormalizerImpl.convert(s);
            char[] value2 = new char[convert.length() * 3];
            int decompose2;
            while (true) {
                decompose2 = NormalizerImpl.decompose(convert.toCharArray(), 0, convert.length(), value2, 0, value2.length, b, array, nx);
                if (decompose2 <= value2.length) {
                    break;
                }
                value2 = new char[decompose2];
            }
            return new String(value2, 0, decompose2);
        }
        char[] value = new char[s.length() * 3];
        int decompose;
        while (true) {
            decompose = NormalizerImpl.decompose(s.toCharArray(), 0, s.length(), value, 0, value.length, b, array, nx);
            if (decompose <= value.length) {
                break;
            }
            value = new char[decompose];
        }
        return new String(value, 0, decompose);
    }
    
    public static int normalize(char[] array, int n, int n2, char[] array2, int n3, int n4, NormalizerBase.Mode mode, int n5) {
        int normalize = mode.normalize(array, n, n2, array2, n3, n4, n5);
        if (normalize <= n4 - n3) {
            return normalize;
        }
        throw new IndexOutOfBoundsException(Integer.toString(normalize));
    }
    
    public int current() {
        if (this.bufferPos < this.bufferLimit || this.nextNormalize()) {
            return this.getCodePointAt(this.bufferPos);
        }
        return -1;
    }
    
    public int next() {
        if (this.bufferPos < this.bufferLimit || this.nextNormalize()) {
            int codePoint = this.getCodePointAt(this.bufferPos);
            this.bufferPos += ((codePoint > 65535) ? 2 : 1);
            return codePoint;
        }
        return -1;
    }
    
    public int previous() {
        if (this.bufferPos > 0 || this.previousNormalize()) {
            int codePoint = this.getCodePointAt(this.bufferPos - 1);
            this.bufferPos -= ((codePoint > 65535) ? 2 : 1);
            return codePoint;
        }
        return -1;
    }
    
    public void reset() {
        this.text.setIndex(0);
        int n = 0;
        this.nextIndex = n;
        this.currentIndex = n;
        this.clearBuffer();
    }
    
    public void setIndexOnly(int currentIndex) {
        this.text.setIndex(currentIndex);
        this.nextIndex = currentIndex;
        this.currentIndex = currentIndex;
        this.clearBuffer();
    }
    
    @Deprecated
    public int setIndex(int indexOnly) {
        this.setIndexOnly(indexOnly);
        return this.current();
    }
    
    @Deprecated
    public int getBeginIndex() {
        return 0;
    }
    
    @Deprecated
    public int getEndIndex() {
        return this.endIndex();
    }
    
    public int getIndex() {
        if (this.bufferPos < this.bufferLimit) {
            return this.currentIndex;
        }
        return this.nextIndex;
    }
    
    public int endIndex() {
        return this.text.getLength();
    }
    
    public void setMode(NormalizerBase.Mode mode) {
        this.mode = mode;
    }
    
    public NormalizerBase.Mode getMode() {
        return this.mode;
    }
    
    public void setText(String s) {
        UCharacterIterator instance = UCharacterIterator.getInstance(s);
        if (instance == null) {
            throw new InternalError("Could not create a new UCharacterIterator");
        }
        this.text = instance;
        this.reset();
        return;
    }
    
    public void setText(CharacterIterator characterIterator) {
        UCharacterIterator instance = UCharacterIterator.getInstance(characterIterator);
        if (instance == null) {
            throw new InternalError("Could not create a new UCharacterIterator");
        }
        this.text = instance;
        int n = 0;
        this.nextIndex = n;
        this.currentIndex = n;
        this.clearBuffer();
        return;
    }
    
    private static long getPrevNorm32(UCharacterIterator uCharacterIterator, int n, int n2, char[] array) {
        int previous;
        if ((previous = uCharacterIterator.previous()) == -1) {
            return 0L;
        }
        array[0] = (char)previous;
        array[1] = '\0';
        if (array[0] < n) {
            return 0L;
        }
        if (!UTF16.isSurrogate(array[0])) {
            return NormalizerImpl.getNorm32(array[0]);
        }
        if (UTF16.isLeadSurrogate(array[0]) || uCharacterIterator.getIndex() == 0) {
            array[1] = (char)uCharacterIterator.current();
            return 0L;
        }
        int n3 = 1;
        char c = (char)uCharacterIterator.previous();
        array[n3] = c;
        if (UTF16.isLeadSurrogate(c)) {
            long norm32 = NormalizerImpl.getNorm32(array[1]);
            if ((norm32 & (long)n2) == 0x0L) {
                return 0L;
            }
            return NormalizerImpl.getNorm32FromSurrogatePair(norm32, array[0]);
        }
        else {
            uCharacterIterator.moveIndex(1);
            return 0L;
        }
    }
    
    private static int findPreviousIterationBoundary(UCharacterIterator uCharacterIterator, NormalizerBase.IsPrevBoundary isPrevBoundary, int n, int n2, char[] array, int[] array2) {
        char[] array3 = new char[2];
        array2[0] = array.length;
        array3[0] = '\0';
        while (uCharacterIterator.getIndex() > 0 && array3[0] != -1) {
            int prevBoundary = isPrevBoundary.isPrevBoundary(uCharacterIterator, n, n2, array3);
            if (array2[0] < ((array3[1] == '\0') ? 1 : 2)) {
                char[] array4 = new char[array.length * 2];
                System.arraycopy(array, array2[0], array4, array4.length - (array.length - array2[0]), array.length - array2[0]);
                int n3 = 0;
                array2[n3] += array4.length - array.length;
                array = array4;
            }
            char[] array5 = array;
            int n4 = 0;
            array5[--array2[n4]] = array3[0];
            if (array3[1] != '\0') {
                char[] array6 = array;
                int n5 = 0;
                array6[--array2[n5]] = array3[1];
            }
            if (!(prevBoundary == 0)) {
                break;
            }
            continue;
        }
        return array.length - array2[0];
    }
    
    private static int previous(UCharacterIterator uCharacterIterator, char[] array, int n, int n2, NormalizerBase.Mode mode, boolean b, boolean[] array2, int n3) {
        int n4 = n2 - n;
        int normalize = 0;
        if (array2 != null) {
            array2[0] = false;
        }
        int n5 = (char)mode.getMinC();
        int mask = mode.getMask();
        NormalizerBase.IsPrevBoundary prevBoundary = mode.getPrevBoundary();
        if (prevBoundary == null) {
            int n6 = 0;
            int previous;
            if ((previous = uCharacterIterator.previous()) >= 0) {
                n6 = 1;
                if (UTF16.isTrailSurrogate((char)previous)) {
                    int previous2 = uCharacterIterator.previous();
                    if (previous2 != -1) {
                        if (UTF16.isLeadSurrogate((char)previous2)) {
                            if (n4 >= 2) {
                                array[1] = (char)previous;
                                n6 = 2;
                            }
                            previous = previous2;
                        }
                        else {
                            uCharacterIterator.moveIndex(1);
                        }
                    }
                }
                if (n4 > 0) {
                    array[0] = (char)previous;
                }
            }
            return n6;
        }
        else {
            char[] array3 = new char[100];
            int[] array4 = { 0 };
            int previousIterationBoundary = findPreviousIterationBoundary(uCharacterIterator, prevBoundary, n5, mask, array3, array4);
            if (previousIterationBoundary > 0) {
                if (b) {
                    normalize = normalize(array3, array4[0], array4[0] + previousIterationBoundary, array, n, n2, mode, n3);
                    if (array2 != null) {
                        array2[0] = (normalize != previousIterationBoundary || Utility.arrayRegionMatches(array3, 0, array, n, n2));
                    }
                }
                else if (n4 > 0) {
                    System.arraycopy(array3, array4[0], array, 0, (previousIterationBoundary < n4) ? previousIterationBoundary : n4);
                }
            }
            return normalize;
        }
    }
    
    private static long getNextNorm32(UCharacterIterator uCharacterIterator, int n, int n2, int[] array) {
        array[0] = uCharacterIterator.next();
        array[1] = 0;
        if (array[0] < n) {
            return 0L;
        }
        long norm32 = NormalizerImpl.getNorm32((char)array[0]);
        if (UTF16.isLeadSurrogate((char)array[0])) {
            if (uCharacterIterator.current() != -1) {
                int n3 = 1;
                int current = uCharacterIterator.current();
                array[n3] = current;
                if (UTF16.isTrailSurrogate((char)current)) {
                    uCharacterIterator.moveIndex(1);
                    if ((norm32 & (long)n2) == 0x0L) {
                        return 0L;
                    }
                    return NormalizerImpl.getNorm32FromSurrogatePair(norm32, (char)array[1]);
                }
            }
            return 0L;
        }
        else {
            return norm32;
        }
    }
    
    private static int findNextIterationBoundary(UCharacterIterator uCharacterIterator, NormalizerBase.IsNextBoundary isNextBoundary, int n, int n2, char[] array) {
        if (uCharacterIterator.current() == -1) {
            return 0;
        }
        int[] array2 = { uCharacterIterator.next(), 0 };
        array[0] = (char)array2[0];
        int n3 = 1;
        if (UTF16.isLeadSurrogate((char)array2[0]) && uCharacterIterator.current() != -1) {
            int n4 = 1;
            int next = uCharacterIterator.next();
            array2[n4] = next;
            if (UTF16.isTrailSurrogate((char)next)) {
                array[n3++] = (char)array2[1];
            }
            else {
                uCharacterIterator.moveIndex(-1);
            }
        }
        while (uCharacterIterator.current() != -1) {
            if (isNextBoundary.isNextBoundary(uCharacterIterator, n, n2, array2)) {
                uCharacterIterator.moveIndex((array2[1] == 0) ? -1 : -2);
                break;
            }
            if (n3 + ((array2[1] == 0) ? 1 : 2) <= array.length) {
                array[n3++] = (char)array2[0];
                if (array2[1] == 0) {
                    continue;
                }
                array[n3++] = (char)array2[1];
            }
            else {
                char[] array3 = new char[array.length * 2];
                System.arraycopy(array, 0, array3, 0, n3);
                array = array3;
                array[n3++] = (char)array2[0];
                if (array2[1] != 0) {
                    array[n3++] = (char)array2[1];
                }
                else {
                    continue;
                }
            }
        }
        return n3;
    }
    
    private static int next(UCharacterIterator uCharacterIterator, char[] array, int n, int n2, NormalizerBase.Mode mode, boolean b, boolean[] array2, int n3) {
        int b2 = n2 - n;
        int normalize = 0;
        if (array2 != null) {
            array2[0] = false;
        }
        int n4 = (char)mode.getMinC();
        int mask = mode.getMask();
        NormalizerBase.IsNextBoundary nextBoundary = mode.getNextBoundary();
        if (nextBoundary == null) {
            int n5 = 0;
            int next = uCharacterIterator.next();
            if (next != -1) {
                n5 = 1;
                if (UTF16.isLeadSurrogate((char)next)) {
                    int next2 = uCharacterIterator.next();
                    if (next2 != -1) {
                        if (UTF16.isTrailSurrogate((char)next2)) {
                            if (b2 >= 2) {
                                array[1] = (char)next2;
                                n5 = 2;
                            }
                        }
                        else {
                            uCharacterIterator.moveIndex(-1);
                        }
                    }
                }
                if (b2 > 0) {
                    array[0] = (char)next;
                }
            }
            return n5;
        }
        else {
            char[] array3 = new char[100];
            int[] array4 = { 0 };
            int nextIterationBoundary = findNextIterationBoundary(uCharacterIterator, nextBoundary, n4, mask, array3);
            if (nextIterationBoundary > 0) {
                if (b) {
                    normalize = mode.normalize(array3, array4[0], nextIterationBoundary, array, n, n2, n3);
                    if (array2 != null) {
                        array2[0] = (normalize != nextIterationBoundary || Utility.arrayRegionMatches(array3, array4[0], array, n, normalize));
                    }
                }
                else if (b2 > 0) {
                    System.arraycopy(array3, 0, array, n, Math.min(nextIterationBoundary, b2));
                }
            }
            return normalize;
        }
    }
    
    private void clearBuffer() {
        int bufferLimit = 0;
        this.bufferPos = bufferLimit;
        this.bufferStart = bufferLimit;
        this.bufferLimit = bufferLimit;
    }
    
    private boolean nextNormalize() {
        this.clearBuffer();
        this.currentIndex = this.nextIndex;
        this.text.setIndex(this.nextIndex);
        this.bufferLimit = next(this.text, this.buffer, this.bufferStart, this.buffer.length, this.mode, true, null, this.options);
        this.nextIndex = this.text.getIndex();
        return this.bufferLimit > 0;
    }
    
    private boolean previousNormalize() {
        this.clearBuffer();
        this.nextIndex = this.currentIndex;
        this.text.setIndex(this.currentIndex);
        this.bufferLimit = previous(this.text, this.buffer, this.bufferStart, this.buffer.length, this.mode, true, null, this.options);
        this.currentIndex = this.text.getIndex();
        this.bufferPos = this.bufferLimit;
        return this.bufferLimit > 0;
    }
    
    private int getCodePointAt(int n) {
        if (UTF16.isSurrogate(this.buffer[n])) {
            if (UTF16.isLeadSurrogate(this.buffer[n])) {
                if (n + 1 < this.bufferLimit && UTF16.isTrailSurrogate(this.buffer[n + 1])) {
                    return UCharacterProperty.getRawSupplementary(this.buffer[n], this.buffer[n + 1]);
                }
            }
            else if (UTF16.isTrailSurrogate(this.buffer[n]) && n > 0 && UTF16.isLeadSurrogate(this.buffer[n - 1])) {
                return UCharacterProperty.getRawSupplementary(this.buffer[n - 1], this.buffer[n]);
            }
        }
        return this.buffer[n];
    }
    
    public static boolean isNFSkippable(int n, NormalizerBase.Mode mode) {
        return mode.isNFSkippable(n);
    }
    
    public NormalizerBase(String s, NormalizerBase.Mode mode) {
        this(s, mode, 0);
    }
    
    public static String normalize(String s, Normalizer.Form form) {
        return normalize(s, form, 0);
    }
    
    public static String normalize(String s, Normalizer.Form obj, int n) {
        int length = s.length();
        boolean n2 = true;
        if (length < 80) {
            int j = 0;
            while (j < length) {
                if (s.charAt(j) > '\u007f') {
                    n2 = false;
                    break;
                }
                ++j;
            }
        }
        else {
            char[] charArray = s.toCharArray();
            int i = 0;
            while (i < length) {
                if (charArray[i] > '\u007f') {
                    n2 = false;
                    break;
                }
                ++i;
            }
        }
        switch (obj) {
            case NFC: {
                return n2 ? s : NormalizerBase.NFC.normalize(s, n);
            }
            case NFD: {
                return n2 ? s : NormalizerBase.NFD.normalize(s, n);
            }
            case NFKC: {
                return n2 ? s : NormalizerBase.NFKC.normalize(s, n);
            }
            case NFKD: {
                return n2 ? s : NormalizerBase.NFKD.normalize(s, n);
            }
            default: {
                throw new IllegalArgumentException("Unexpected normalization form: " + obj);
            }
        }
    }
    
    public static boolean isNormalized(String s, Normalizer.Form form) {
        return isNormalized(s, form, 0);
    }
    
    public static boolean isNormalized(String s, Normalizer.Form obj, int n) {
        switch (obj) {
            case NFC: {
                return NormalizerBase.NFC.quickCheck(s.toCharArray(), 0, s.length(), false, NormalizerImpl.getNX(n)) == NormalizerBase.YES;
            }
            case NFD: {
                return NormalizerBase.NFD.quickCheck(s.toCharArray(), 0, s.length(), false, NormalizerImpl.getNX(n)) == NormalizerBase.YES;
            }
            case NFKC: {
                return NormalizerBase.NFKC.quickCheck(s.toCharArray(), 0, s.length(), false, NormalizerImpl.getNX(n)) == NormalizerBase.YES;
            }
            case NFKD: {
                return NormalizerBase.NFKD.quickCheck(s.toCharArray(), 0, s.length(), false, NormalizerImpl.getNX(n)) == NormalizerBase.YES;
            }
            default: {
                throw new IllegalArgumentException("Unexpected normalization form: " + obj);
            }
        }
    }
    
    static long access$1000(UCharacterIterator uCharacterIterator, int n, int n2, char[] array) {
        return getPrevNorm32(uCharacterIterator, n, n2, array);
    }
    
    static long access$1100(UCharacterIterator uCharacterIterator, int n, int n2, int[] array) {
        return getNextNorm32(uCharacterIterator, n, n2, array);
    }
    
    static {
        NONE = new NormalizerBase.Mode(1);
        NFD = new NormalizerBase.NFDMode(2);
        NFKD = new NormalizerBase.NFKDMode(3);
        NFC = new NormalizerBase.NFCMode(4);
        NFKC = new NormalizerBase.NFKCMode(5);
        NO = new NormalizerBase.QuickCheckResult(0);
        YES = new NormalizerBase.QuickCheckResult(1);
        MAYBE = new NormalizerBase.QuickCheckResult(2);
    }
    
    public static class Mode
    {
        private int modeValue;
        
        private Mode(int modeValue) {
            this.modeValue = modeValue;
        }
        
        protected int normalize(char[] array, int n, int n2, char[] array2, int n3, int n4, UnicodeSet set) {
            int n5 = n2 - n;
            if (n5 > n4 - n3) {
                return n5;
            }
            System.arraycopy(array, n, array2, n3, n5);
            return n5;
        }
        
        protected int normalize(char[] array, int n, int n2, char[] array2, int n3, int n4, int n5) {
            return this.normalize(array, n, n2, array2, n3, n4, NormalizerImpl.getNX(n5));
        }
        
        protected String normalize(String s, int n) {
            return s;
        }
        
        protected int getMinC() {
            return -1;
        }
        
        protected int getMask() {
            return -1;
        }
        
        protected NormalizerBase.IsPrevBoundary getPrevBoundary() {
            return null;
        }
        
        protected NormalizerBase.IsNextBoundary getNextBoundary() {
            return null;
        }
        
        protected NormalizerBase.QuickCheckResult quickCheck(char[] array, int n, int n2, boolean b, UnicodeSet set) {
            if (b) {
                return NormalizerBase.MAYBE;
            }
            return NormalizerBase.NO;
        }
        
        protected boolean isNFSkippable(int n) {
            return true;
        }
    }
    
    private static final class NFDMode extends NormalizerBase.Mode
    {
        private NFDMode(int n) {
            super(n);
        }
        
        @Override
        protected int normalize(char[] array, int n, int n2, char[] array2, int n3, int n4, UnicodeSet set) {
            return NormalizerImpl.decompose(array, n, n2, array2, n3, n4, false, new int[1], set);
        }
        
        @Override
        protected String normalize(String s, int n) {
            return NormalizerBase.decompose(s, false, n);
        }
        
        @Override
        protected int getMinC() {
            return 768;
        }
        
        @Override
        protected NormalizerBase.IsPrevBoundary getPrevBoundary() {
            return new NormalizerBase.IsPrevNFDSafe();
        }
        
        @Override
        protected NormalizerBase.IsNextBoundary getNextBoundary() {
            return new NormalizerBase.IsNextNFDSafe();
        }
        
        @Override
        protected int getMask() {
            return 65284;
        }
        
        @Override
        protected NormalizerBase.QuickCheckResult quickCheck(char[] array, int n, int n2, boolean b, UnicodeSet set) {
            return NormalizerImpl.quickCheck(array, n, n2, NormalizerImpl.getFromIndexesArr(8), 4, 0, b, set);
        }
        
        @Override
        protected boolean isNFSkippable(int n) {
            return NormalizerImpl.isNFSkippable(n, this, 65284L);
        }
    }
    
    private static final class NFKDMode extends NormalizerBase.Mode
    {
        private NFKDMode(int n) {
            super(n);
        }
        
        @Override
        protected int normalize(char[] array, int n, int n2, char[] array2, int n3, int n4, UnicodeSet set) {
            return NormalizerImpl.decompose(array, n, n2, array2, n3, n4, true, new int[1], set);
        }
        
        @Override
        protected String normalize(String s, int n) {
            return NormalizerBase.decompose(s, true, n);
        }
        
        @Override
        protected int getMinC() {
            return 768;
        }
        
        @Override
        protected NormalizerBase.IsPrevBoundary getPrevBoundary() {
            return new NormalizerBase.IsPrevNFDSafe();
        }
        
        @Override
        protected NormalizerBase.IsNextBoundary getNextBoundary() {
            return new NormalizerBase.IsNextNFDSafe();
        }
        
        @Override
        protected int getMask() {
            return 65288;
        }
        
        @Override
        protected NormalizerBase.QuickCheckResult quickCheck(char[] array, int n, int n2, boolean b, UnicodeSet set) {
            return NormalizerImpl.quickCheck(array, n, n2, NormalizerImpl.getFromIndexesArr(9), 8, 4096, b, set);
        }
        
        @Override
        protected boolean isNFSkippable(int n) {
            return NormalizerImpl.isNFSkippable(n, this, 65288L);
        }
    }
    
    private static final class NFCMode extends NormalizerBase.Mode
    {
        private NFCMode(int n) {
            super(n);
        }
        
        @Override
        protected int normalize(char[] array, int n, int n2, char[] array2, int n3, int n4, UnicodeSet set) {
            return NormalizerImpl.compose(array, n, n2, array2, n3, n4, 0, set);
        }
        
        @Override
        protected String normalize(String s, int n) {
            return NormalizerBase.compose(s, false, n);
        }
        
        @Override
        protected int getMinC() {
            return NormalizerImpl.getFromIndexesArr(6);
        }
        
        @Override
        protected NormalizerBase.IsPrevBoundary getPrevBoundary() {
            return new NormalizerBase.IsPrevTrueStarter();
        }
        
        @Override
        protected NormalizerBase.IsNextBoundary getNextBoundary() {
            return new NormalizerBase.IsNextTrueStarter();
        }
        
        @Override
        protected int getMask() {
            return 65297;
        }
        
        @Override
        protected NormalizerBase.QuickCheckResult quickCheck(char[] array, int n, int n2, boolean b, UnicodeSet set) {
            return NormalizerImpl.quickCheck(array, n, n2, NormalizerImpl.getFromIndexesArr(6), 17, 0, b, set);
        }
        
        @Override
        protected boolean isNFSkippable(int n) {
            return NormalizerImpl.isNFSkippable(n, this, 65473L);
        }
    }
    
    private static final class NFKCMode extends NormalizerBase.Mode
    {
        private NFKCMode(int n) {
            super(n);
        }
        
        @Override
        protected int normalize(char[] array, int n, int n2, char[] array2, int n3, int n4, UnicodeSet set) {
            return NormalizerImpl.compose(array, n, n2, array2, n3, n4, 4096, set);
        }
        
        @Override
        protected String normalize(String s, int n) {
            return NormalizerBase.compose(s, true, n);
        }
        
        @Override
        protected int getMinC() {
            return NormalizerImpl.getFromIndexesArr(7);
        }
        
        @Override
        protected NormalizerBase.IsPrevBoundary getPrevBoundary() {
            return new NormalizerBase.IsPrevTrueStarter();
        }
        
        @Override
        protected NormalizerBase.IsNextBoundary getNextBoundary() {
            return new NormalizerBase.IsNextTrueStarter();
        }
        
        @Override
        protected int getMask() {
            return 65314;
        }
        
        @Override
        protected NormalizerBase.QuickCheckResult quickCheck(char[] array, int n, int n2, boolean b, UnicodeSet set) {
            return NormalizerImpl.quickCheck(array, n, n2, NormalizerImpl.getFromIndexesArr(7), 34, 4096, b, set);
        }
        
        @Override
        protected boolean isNFSkippable(int n) {
            return NormalizerImpl.isNFSkippable(n, this, 65474L);
        }
    }
    
    public static final class QuickCheckResult
    {
        private int resultValue;
        
        private QuickCheckResult(int resultValue) {
            this.resultValue = resultValue;
        }
    }
    
    private static final class IsPrevNFDSafe implements NormalizerBase.IsPrevBoundary
    {
        @Override
        public boolean isPrevBoundary(UCharacterIterator uCharacterIterator, int n, int n2, char[] array) {
            return NormalizerImpl.isNFDSafe(NormalizerBase.access$1000(uCharacterIterator, n, n2, array), n2, n2 & 0x3F);
        }
    }
    
    private static final class IsPrevTrueStarter implements NormalizerBase.IsPrevBoundary
    {
        @Override
        public boolean isPrevBoundary(UCharacterIterator uCharacterIterator, int n, int n2, char[] array) {
            int n3 = n2 << 2 & 0xF;
            return NormalizerImpl.isTrueStarter(NormalizerBase.access$1000(uCharacterIterator, n, n2 | n3, array), n2, n3);
        }
    }
    
    private static final class IsNextNFDSafe implements NormalizerBase.IsNextBoundary
    {
        @Override
        public boolean isNextBoundary(UCharacterIterator uCharacterIterator, int n, int n2, int[] array) {
            return NormalizerImpl.isNFDSafe(NormalizerBase.access$1100(uCharacterIterator, n, n2, array), n2, n2 & 0x3F);
        }
    }
    
    private static final class IsNextTrueStarter implements NormalizerBase.IsNextBoundary
    {
        @Override
        public boolean isNextBoundary(UCharacterIterator uCharacterIterator, int n, int n2, int[] array) {
            int n3 = n2 << 2 & 0xF;
            return NormalizerImpl.isTrueStarter(NormalizerBase.access$1100(uCharacterIterator, n, n2 | n3, array), n2, n3);
        }
    }
    
    private interface IsNextBoundary
    {
        public abstract boolean isNextBoundary(UCharacterIterator p0, int p1, int p2, int[] p3);
    }
    
    private interface IsPrevBoundary
    {
        public abstract boolean isPrevBoundary(UCharacterIterator p0, int p1, int p2, char[] p3);
    }
}
